package com.nisum.attendance.attendance.controller;

import com.nisum.attendance.attendance.exceptions.ResourceNotFoundException;
import com.nisum.attendance.attendance.model.Attendance;
import com.nisum.attendance.attendance.repository.AttendanceRepository;
import com.nisum.attendance.attendance.service.AttendanceService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.List;
import java.util.Optional;

@CrossOrigin(origins = "*")
@RestController
@RequestMapping("/api")
public class AttendanceController {
    @Autowired AttendanceService attendanceService;

    @GetMapping("/attendance")
    public List<Attendance> getAllAttendances() {
        return attendanceService.findAllAttendance();
    }

    @GetMapping("/attendance/{id}")
    public ResponseEntity<Attendance> getAttendance(@PathVariable(value = "id") Long attendanceId) throws ResourceNotFoundException {
        Attendance attendance = attendanceService.getAttendanceById(attendanceId)
                .orElseThrow(() -> new ResourceNotFoundException("Attendance records by id not found." + attendanceId));
        return ResponseEntity.ok(attendance);
    }

    @GetMapping("/attendance/student/{studentEmail}")
    public List<Attendance> getStudentAttendance(@PathVariable(value = "studentEmail") String studentEmail) {
        return attendanceService.getAttendanceByEmail(studentEmail);
    }

    @PostMapping("/attendance")
    public Attendance createAttendance(@Valid @RequestBody Attendance attendance) {
        return attendanceService.addAttendance(attendance);
    }

    @PutMapping("/attendance/{id}")
    public ResponseEntity<Attendance> updateAttendance(@PathVariable(value = "id") Long attendanceId,
                                                       @Valid @RequestBody Attendance attendanceDetails)
                                                        throws ResourceNotFoundException {
        Optional<Attendance> attendanceOptional = attendanceService.getAttendanceById(attendanceId);
        if (attendanceOptional.isPresent()) {
            Attendance attendance = attendanceOptional.get();
        } else {
            throw new ResourceNotFoundException("Attendance record not found." + attendanceId);
        }

        Attendance updatedAttendance = attendanceService.updateAttendance(attendanceOptional.get(), attendanceDetails);

        return ResponseEntity.ok(updatedAttendance);
    }
}
